using System;
using System.Data.Common;
using System.Windows.Forms;
using Microsoft.Win32;

namespace freeSQL
{
    public partial class frmConnect : Form
    {
        string providerName;
        public string ProviderName
        {
            get { return providerName; }
        }

        string connectionString = "";
        public string ConnectionString
        {
            get { return connectionString; }
        }

        public frmConnect()
        {
            InitializeComponent();
        }

        private void frmConnect_Load(object sender, EventArgs e)
        {
            RegistryKey rk = Registry.CurrentUser.OpenSubKey("Software\\FreeSQL");
            if (rk != null)
            {
                cbDbType.Text = rk.GetValue("dbType", "").ToString();
                txtUser.Text = rk.GetValue("dbUser", "").ToString();
                txtServer.Text = rk.GetValue("dbServer", "").ToString();
                txtDataBase.Text = rk.GetValue("dbDataBase", "").ToString();
            }

            if (rk != null)
            {
                string[] v = rk.GetSubKeyNames();
                for (int i = 0; i < v.Length; i++)
                    cbProfile.Items.Add(v[i]);
            }
        }
        private void frmConnect_Activated(object sender, EventArgs e)
        {
            if (txtUser.Text != "")
                txtPassword.Focus();
        }

        private void btnOk_Click(object sender, EventArgs e)
        {
            this.Enabled = false;

            Cursor.Current = Cursors.WaitCursor;
            Application.DoEvents();

            if (cbDbType.Text == "Microsoft Access")
            {
                providerName = "System.Data.OleDb";
                connectionString = "Provider=Microsoft.Jet.OLEDB.4.0; Data Source=" + txtDataBase.Text + "; Jet OLEDB:Database Password=" + txtPassword.Text;
            }
            else if (cbDbType.Text == "Microsoft SQL Server")
            {
                providerName = "System.Data.SqlClient";
                connectionString = "Data Source=" + txtServer.Text + "; DataBase=" + txtDataBase.Text + "; User ID=" + txtUser.Text + "; Password=" + txtPassword.Text;
            }
            else if (cbDbType.Text == "Oracle 7")
            {
                providerName = "System.Data.OleDb";
                connectionString = "Provider=MSDAORA; Data Source=" + txtServer.Text + "; User ID=" + txtUser.Text + "; Password=" + txtPassword.Text;
            }
            else if (cbDbType.Text == "Oracle 8 and later")
            {
                providerName = "System.Data.OracleClient";
                connectionString = "Data Source=" + txtServer.Text + "; User ID=" + txtUser.Text + "; Password=" + txtPassword.Text;
            }
            else if (cbDbType.Text == "MySQL")
            {
                providerName = "MySql.Data.MySqlClient";
                connectionString = "Server=" + txtServer.Text + "; DataBase=" + txtDataBase.Text + "; User ID=" + txtUser.Text + "; Password=" + txtPassword.Text;
            }

            try
            {
                DbProviderFactory pf = DbProviderFactories.GetFactory(providerName);
                DbConnection cn = pf.CreateConnection();
                cn.ConnectionString = connectionString;
                cn.Open();
                cn.Close();
            }
            catch (Exception ex)
            {
                this.Enabled = true;
                Cursor.Current = null;

                MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            RegistryKey rk = Registry.CurrentUser.CreateSubKey("Software\\FreeSQL");
            rk.SetValue("dbType", cbDbType.Text, RegistryValueKind.String);
            rk.SetValue("dbUser", txtUser.Text, RegistryValueKind.String);
            rk.SetValue("dbServer", txtServer.Text, RegistryValueKind.String);
            rk.SetValue("dbDataBase", txtDataBase.Text, RegistryValueKind.String);

            if (cbProfile.Text != "")
            {
                rk = rk.CreateSubKey(cbProfile.Text);
                rk.SetValue("dbType", cbDbType.Text, RegistryValueKind.String);
                rk.SetValue("dbUser", txtUser.Text, RegistryValueKind.String);
                rk.SetValue("dbPassword", txtPassword.Text, RegistryValueKind.String);
                rk.SetValue("dbServer", txtServer.Text, RegistryValueKind.String);
                rk.SetValue("dbDataBase", txtDataBase.Text, RegistryValueKind.String);
            }
            this.Close();
        }
        private void btnCancel_Click(object sender, EventArgs e)
        {
            providerName = "";
            connectionString = "";
            this.Close();
        }
        private void btnHelp_Click(object sender, EventArgs e)
        {
            netlib.netlib7 n7 = new netlib.netlib7();

            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(freeSQL.Properties.Resources));
            string helpText = resources.GetString("helpText");
            helpText = helpText.Replace("$version", Application.ProductVersion);
            helpText = helpText.Replace("$date", System.IO.File.GetCreationTime(Application.ExecutablePath).ToString("D"));
            n7.FileWrite(Environment.GetEnvironmentVariable("TEMP") + "\\freeSQL.htm", helpText, false);
            System.Diagnostics.Process p = new System.Diagnostics.Process();
            p.StartInfo.FileName = Environment.GetEnvironmentVariable("TEMP") + "\\freeSQL.htm";
            p.StartInfo.UseShellExecute = true;
            p.Start();
        }
        private void btnBrowse_Click(object sender, EventArgs e)
        {
            cbProfile.Text = "";
            OpenFileDialog f = new OpenFileDialog();
            if (txtDataBase.Text != "")
                f.InitialDirectory = System.IO.Path.GetDirectoryName(txtDataBase.Text);
            f.Filter = "Microsoft Access Database|*.mdb";
            f.ShowDialog();
            if (f.FileName != "") txtDataBase.Text = f.FileName;
        }
        private void btnRemoveProfile_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show("Do you really want to remove selected profile?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
            if (dr == DialogResult.No) return;

            if (cbProfile.Text != "")
            {
                Registry.CurrentUser.DeleteSubKeyTree("Software\\FreeSQL\\" + cbProfile.Text);
                cbProfile.Items.Remove(cbProfile.Text);
            }
        }

        private void cbProfile_SelectedIndexChanged(object sender, EventArgs e)
        {
            netlib.netlib7 n7 = new netlib.netlib7();

            RegistryKey rk = Registry.CurrentUser.OpenSubKey("Software\\FreeSQL\\" + cbProfile.Text);
            if (rk != null)
            {
                cbDbType.Text = n7.nvl(rk.GetValue("dbType"));
                txtUser.Text = n7.nvl(rk.GetValue("dbUser"));
                txtPassword.Text = n7.nvl(rk.GetValue("dbPassword"));
                txtServer.Text = n7.nvl(rk.GetValue("dbServer"));
                txtDataBase.Text = n7.nvl(rk.GetValue("dbDataBase"));
            }
        }
        private void cbProfile_TextChanged(object sender, EventArgs e)
        {
            if (cbProfile.Text == "")
                btnRemoveProfile.Visible = false;
            else
                btnRemoveProfile.Visible = true;
        }
        private void cbDbType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cbDbType.Text == "Microsoft Access")
            {
                txtUser.Enabled = false;
                txtServer.Enabled = false;
                txtDataBase.Enabled = true;
                btnBrowse.Visible = true;
            }
            else if (cbDbType.Text.StartsWith("Oracle"))
            {
                txtUser.Enabled = true;
                txtServer.Enabled = true;
                txtDataBase.Enabled = false;
                btnBrowse.Visible = false;
            }
            else
            {
                txtUser.Enabled = true;
                txtServer.Enabled = true;
                txtDataBase.Enabled = true;
                btnBrowse.Visible = false;
            }
        }
        private void cbDbType_MouseUp(object sender, MouseEventArgs e)
        {
            cbProfile.Text = "";
        }
        private void txtUser_KeyUp(object sender, KeyEventArgs e)
        {
            cbProfile.Text = "";
        }
        private void txtPassword_KeyUp(object sender, KeyEventArgs e)
        {
            cbProfile.Text = "";
        }
        private void txtServer_KeyUp(object sender, KeyEventArgs e)
        {
            cbProfile.Text = "";
        }
        private void txtDataBase_KeyUp(object sender, KeyEventArgs e)
        {
            cbProfile.Text = "";
        }

    }
}