using System;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Printing;
using Microsoft.Win32;

namespace freeSQL
{
    public partial class frmEdit : Form
    {
        static int findPos;
        string originalTextValue;
        public bool readOnly;
        public string textValue
        {
            get { return textBoxForEdit.Text; }
            set { textBoxForEdit.Text = value; }
        }

        public frmEdit()
        {
            InitializeComponent();
        }

        private void frmEdit_Load(object sender, EventArgs e)
        {
            originalTextValue = textValue;
            textValue = textValue.Replace("\r\n", "\n");
            textValue = textValue.Replace("\n", "\r\n");
            textBoxForEdit.ReadOnly = readOnly;
            toolStripButtonOk.Enabled = !readOnly;
            toolStripButtonUndo.Enabled = !readOnly;

            netlib.netlib7 n7 = new netlib.netlib7();

            FontConverter fConv = new FontConverter();
            string editFont = n7.nvl(Registry.GetValue("HKEY_CURRENT_USER\\Software\\FreeSQL", "optFontEdit", ""));
            if (editFont != "") textBoxForEdit.Font = (Font)fConv.ConvertFromString(editFont);

        }
        private void frmEdit_Activated(object sender, EventArgs e)
        {
            textBoxForEdit.Focus();
            textBoxForEdit.DeselectAll();
        }
        private void frmEdit_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Escape)
            {
                textBoxForEdit.Text = originalTextValue;
                this.Close();
            }
        }

        private void toolStripButtonOk_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        private void toolStripButtonUndo_Click(object sender, EventArgs e)
        {
            textBoxForEdit.Text = originalTextValue;
        }
        private void toolStripButtonSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog s = new SaveFileDialog();
            s.Filter = "Text files|*.txt";
            s.ShowDialog();
            if (s.FileName != "")
            {
                StreamWriter w = new StreamWriter(s.FileName);
                w.Write(textBoxForEdit.Text);
                w.Close();
            }
        }

        StreamReader streamToPrint;
        private void toolStripButtonPrint_Click(object sender, EventArgs e)
        {
            MemoryStream m = new MemoryStream(System.Text.Encoding.UTF8.GetBytes(textBoxForEdit.Text));
            streamToPrint = new StreamReader(m);

            PrintDocument printDoc = new PrintDocument();
            printDoc.DocumentName = this.Text;
            printDoc.PrintPage += new PrintPageEventHandler(printDoc_PrintPage);

            printDoc.Print();
        }
        private void printDoc_PrintPage(Object sender, PrintPageEventArgs ev)
        {
            Font printFont = textBoxForEdit.Font;
            float linesPerPage = ev.MarginBounds.Height / printFont.GetHeight(ev.Graphics);
            float yPos = 0;
            int count = 0;
            float leftMargin = ev.MarginBounds.Left;
            float topMargin = ev.MarginBounds.Top;
            string line = null;

            // Print each line of the file.
            while (count < linesPerPage &&
               ((line = streamToPrint.ReadLine()) != null))
            {
                yPos = topMargin + (count *
                   printFont.GetHeight(ev.Graphics));
                ev.Graphics.DrawString(line, printFont, Brushes.Black,
                   leftMargin, yPos, new StringFormat());
                count++;
            }

            // If more lines exist, print another page.
            if (line != null)
                ev.HasMorePages = true;
            else
                ev.HasMorePages = false;
        }

        private void toolStripButtonWrap_Click(object sender, EventArgs e)
        {
            textBoxForEdit.WordWrap = (!textBoxForEdit.WordWrap);
            toolStripButtonWrap.Checked = (!toolStripButtonWrap.Checked);
        }
        private void textBoxForFind_TextChanged(object sender, EventArgs e)
        {
            findPos = 0;
        }
        private void toolStripButtonFind_Click(object sender, EventArgs e)
        {
            int p = textBoxForEdit.Text.IndexOf(textBoxForFind.Text, findPos + 1, StringComparison.OrdinalIgnoreCase);
            if (p < 0)
            {
                findPos = 0;
            }
            else
            {
                textBoxForEdit.Select(p, textBoxForFind.Text.Length);
                textBoxForEdit.ScrollToCaret();
                findPos = p;
            }
        }
    }
}