using System;
using System.Data;
using System.Data.Common;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Microsoft.Win32;
using netlib;
using System.Collections;

namespace freeSQL
{
    public partial class frmSQL : Form
    {
        netlib7 n7 = new netlib7();
        DbParameter[] parameters;
        int showAllObj = 2;
        bool sqlTextModified = false;
        bool queryOnlyMode;
        string lastSql = "";
        string sqlName = "";
        bool cancel = false;

        public frmSQL(string ProviderName, string ConnectionString, string mode)
        {
            if (mode.ToLower() == "-plus")
                queryOnlyMode = false;
            else
                queryOnlyMode = true;
            n7.ProviderName = ProviderName;
            n7.ConnectionString = ConnectionString;

            InitializeComponent();

            this.Width = SystemInformation.PrimaryMonitorSize.Width - SystemInformation.PrimaryMonitorSize.Width * 10 / 100;
            this.Height = SystemInformation.PrimaryMonitorSize.Height - SystemInformation.PrimaryMonitorSize.Height * 20 / 100;
        }
        private void frmSQL_Load(object sender, EventArgs e)
        {
            if (n7.ConnectionString == "")
                ChooseDB();
            if (n7.ConnectionString == "")
                this.Close();

            if (queryOnlyMode)
            {
                btnCommitMenu.Enabled = false;
                btnImport.Enabled = false;
                btnExecuteScript.Enabled = false;
                dataGrid.ReadOnly = true;
                dataGrid.AllowUserToDeleteRows = false;

                bindingNavigatorAddNewItem.Visible = false;
                bindingNavigatorDupRecItem.Visible = false;
                bindingNavigatorDeleteItem.Visible = false;
            }

            n7.DataTableToHtmlElapsed += new EventHandler(n7_DataTableToHtmlElapsed);
        }
        void n7_DataTableToHtmlElapsed(object o, EventArgs e)
        {
            ToolStripProgressBar p = (ToolStripProgressBar)statusBar.Items["pBar"];
            p.Value++;
            this.Cursor = Cursors.AppStarting;
            Application.DoEvents();
        }
        private void frmSQL_Activated(object sender, EventArgs e)
        {
            if (Convert.ToInt32(Registry.GetValue("HKEY_CURRENT_USER\\Software\\FreeSQL", "optShowAllObj", 0)) != showAllObj)
            {
                showAllObj = Convert.ToInt32(Registry.GetValue("HKEY_CURRENT_USER\\Software\\FreeSQL", "optShowAllObj", 0));
                ResetTree();
            }
            FontConverter fConv = new FontConverter();
            string dbTreeViewFont = n7.nvl(Registry.GetValue("HKEY_CURRENT_USER\\Software\\FreeSQL", "optFontExplorer", ""));
            string sqlTextFont = n7.nvl(Registry.GetValue("HKEY_CURRENT_USER\\Software\\FreeSQL", "optFontSql", ""));
            string gridFont = n7.nvl(Registry.GetValue("HKEY_CURRENT_USER\\Software\\FreeSQL", "optFontGrid", ""));
            if (dbTreeViewFont != "") dbTreeView.Font = (Font)fConv.ConvertFromString(dbTreeViewFont);
            if (sqlTextFont != "") sqlText.Font = (Font)fConv.ConvertFromString(sqlTextFont);
            if (gridFont != "") dataGrid.Font = (Font)fConv.ConvertFromString(gridFont);

            sqlText.Focus();
        }
        private void frmSQL_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (sqlText.Text != "" && MessageBox.Show("Do you really want to quit?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                e.Cancel = true;
        }
        private void frmSQL_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyData == (Keys.Control | Keys.K))
                btnConnect_Click(null, null);
            else if (e.KeyData == (Keys.Control | Keys.G))
                btnSizeGrid_Click(null, null);
            else if (e.KeyData == Keys.Escape)
                btnStop_Click(null, null);
            else if (e.KeyData == (Keys.F2))
                btnNewQuery_Click(null, null);
            else if (e.KeyData == (Keys.Control | Keys.F12))
                EasterEgg1();
            else if (e.KeyData == (Keys.Alt | Keys.F2))
                EditCell(dataGrid.CurrentCell.RowIndex, dataGrid.CurrentCell.ColumnIndex);
        }

        private void EasterEgg1()
        {
            DbConnection cn = n7.CreateConnection();
            DataTable dt = new DataTable();
            cn.Open();
            string s;
            if (sqlText.SelectedText != "")
                s = sqlText.SelectedText;
            else
                s = QueryCleanUp(sqlText.Text, true);
            string[] v = null;
            int i = s.IndexOf(',');
            if (i > 0)
            {
                v = s.Substring(i + 1).Split(',');
                for (int k = 0; k < v.Length; k++)
                    if (v[k] == "")
                        v[k] = null;
                s = s.Substring(0, i);
            }

            try
            {
                if (v != null)
                    dt = cn.GetSchema(s, v);
                else if (s != "")
                    dt = cn.GetSchema(s);
                else
                    dt = cn.GetSchema();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                cn.Close();
            }
            dataGrid.DataSource = dt;
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            SetStatusText("Connecting...");
            ChooseDB();
            ResetTree();
            bindingSource.DataSource = null;
            SetStatusText("");
        }
        private void btnBrowseData_Click(object sender, EventArgs e)
        {
            if (dbTreeView.SelectedNode == null || n7.nvl(dbTreeView.SelectedNode.Tag) == "")
            {
                MessageBox.Show("Please select an object first.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            sqlTextModified = false;
            sqlText.Text = BuildQuery(dbTreeView.SelectedNode.Text, false);
            btnExecuteQuery_Click(sender, e);
        }
        private void btnHistory_Click(object sender, EventArgs e)
        {
            frmRecall f = new frmRecall();
            f.ShowDialog();
            if (f.ReturnValue != "")
                sqlText.Text = f.ReturnValue;
            SetStatusText("");
        }
        private void btnImport_Click(object sender, EventArgs e)
        {
            int reccount = 0;

            string tableName = "";
            if (dbTreeView.SelectedNode != null && dbTreeView.SelectedNode.Tag != null)
                tableName = dbTreeView.SelectedNode.Text;

            DataTable dt = new DataTable();

            frmImport fimp = new frmImport(tableName);
            fimp.ShowDialog(this);
            if (fimp.retVal == null) return;
            string fileName = fimp.retVal[0];
            char delimiter = (fimp.retVal[1] == "(TAB)" ? Convert.ToChar(9) : fimp.retVal[1].ToCharArray()[0]);
            string firstRow = fimp.retVal[2];

            if (n7.IsIn(Path.GetExtension(fileName), ".txt,.csv,.dat"))
            {
                bool kcol = false;
                string[] rows, cols, colsValue;

                StreamReader f = new StreamReader(fileName);
                string s = f.ReadToEnd();
                s = s.Replace('\r', '\n');
                s = s.Replace("\n\n", "\n");
                rows = s.Split('\n');
                f.Close();
                cols = rows[0].Split(delimiter);

                if (tableName == "")
                {
                    if (firstRow == "F")
                    {
                        MessageBox.Show("The file has no column names on the first row. You must choose a table before import.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                        return;
                    }
                    tableName = Path.GetFileNameWithoutExtension(fileName).Replace('-', '_').Replace(' ', '_');
                    tableName = n7.StringFilter(tableName, netlib7.StringType.CharsAndNumbersAndSpaces, "_");

                    if (MessageBox.Show("Continue?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                        return;

                    int i = BuildCreateTable(tableName, cols);
                    if (i < 0)
                        return;
                    else if (i == 0)
                    {
                        n7.ExecNonQuery("ALTER TABLE " + tableName + " ADD keyCol VARCHAR(10) PRIMARY KEY");
                        kcol = true;
                    }
                }
                n7.DataTableFill(dt, "select * from " + tableName + " where 1=2 ");

                SetStatusText("Importing data...");
                this.Enabled = false;
                for (int i = (firstRow == "F" ? 0 : 1); i < rows.Length; i++)
                {
                    if (rows[i].Trim() != "")
                    {
                        DataRow rw = dt.NewRow();
                        colsValue = rows[i].Split(delimiter);
                        for (int k = 0; k < cols.Length; k++)
                            rw[k] = colsValue[k].Trim();
                        if (kcol)
                            rw["keyCol"] = i.ToString().PadLeft(10, '0');
                        dt.Rows.Add(rw);
                    }
                }
                this.Enabled = true;
                reccount = rows.Length;
            }
            else if (Path.GetExtension(fileName).ToLower() == ".xml")
            {
                DataSet ds = new DataSet();
                ds.ReadXml(fileName);
                dt = ds.Tables[0];
                reccount = dt.Rows.Count;

                if (tableName == "")
                {
                    string[] cols = new string[dt.Columns.Count];
                    for (int i = 0; i < dt.Columns.Count; i++)
                        cols[i] = dt.Columns[i].ColumnName;

                    tableName = Path.GetFileNameWithoutExtension(fileName).Replace('-', '_').Replace(' ', '_');
                    tableName = n7.StringFilter(tableName, netlib7.StringType.CharsAndNumbersAndSpaces, "_");

                    if (MessageBox.Show("Continue?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                        return;
                    if (BuildCreateTable(tableName, cols) < 0)
                        return;
                }
            }
            else
            {
                MessageBox.Show("Format not recognized", Application.ProductName);
                return;
            }

            if (MessageBox.Show(reccount.ToString() + " record(s) imported. Do you want to commit?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                return;

            SetStatusText("Updating database in progress please wait...");
            n7.DataTableUpdate(dt, "select * from " + tableName + " where 1=2 ");
            SetStatusText("");

            if (n7.ErrorMessage != "")
                MessageBox.Show(n7.ErrorMessage, Application.ProductName);
            else
                MessageBox.Show("Import completed", Application.ProductName);
        }
        private void btnOptions_Click(object sender, EventArgs e)
        {
            frmOptions f = new frmOptions();
            f.ShowDialog(this);
            f.Dispose();
        }
        private void btnRefresh_Click(object sender, EventArgs e)
        {
            ResetTree();
        }
        private void btnExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnConvertForCSharp_Click(object sender, EventArgs e)
        {
            char[] c = { '\r', '\n' };
            string r = "";
            if (sqlText.Text != "")
            {
                string[] v = sqlText.Text.Split(c);

                for (int i = 0; i < v.Length; i++)
                {
                    if (v[i] != "")
                    {
                        if (i == 0)
                            r = "q = \"" + v[i] + "\"\r\n";
                        else
                            r += "  + \"" + v[i] + "\"\r\n";
                    }
                }
                r = n7.RTrunc(r, 2) + ";";
            }
            if (r != "")
                Clipboard.SetText(r, TextDataFormat.Text);
            SetStatusText("Query text copied into the clipboard for use with C#");
        }
        private void btnBeautify_Click(object sender, EventArgs e)
        {
            try
            {
                string s;
                int startPosition, textLength;

                if (sqlText.SelectedText != "")
                {
                    s = sqlText.SelectedText;
                    if (s.IndexOf("\r\n/") >= 0) return;

                    startPosition = sqlText.SelectionStart;
                    textLength = sqlText.SelectionLength;
                }
                else if (sqlText.Text.IndexOf("\r\n/") > 0)
                {
                    int actualPosition = sqlText.SelectionStart;
                    startPosition = sqlText.Text.LastIndexOf("\r\n/", actualPosition);
                    if (startPosition < 0)
                        startPosition = 0;
                    else
                        startPosition = startPosition + 5;
                    int endPosition = sqlText.Text.IndexOf("\r\n/", actualPosition);
                    if (endPosition < 0) endPosition = sqlText.Text.Length;

                    s = sqlText.Text.Substring(startPosition, endPosition - startPosition);
                    textLength = s.Length;

                    s = s.Replace("\r\n/\r\n", "");
                    s = s.Replace("\r\n/", "");
                    s = s.Replace("/\r\n", "");
                }
                else
                {
                    s = sqlText.Text;
                    startPosition = 0;
                    textLength = s.Length;
                }

                s = QueryCleanUp(s, true);
                if (s.StartsWith("SELECT", StringComparison.OrdinalIgnoreCase))
                    s = BeautifySelDel("SELECT", s, "");
                else if (s.StartsWith("INSERT", StringComparison.OrdinalIgnoreCase))
                    s = BeautifyInsert(s);
                else if (s.StartsWith("UPDATE", StringComparison.OrdinalIgnoreCase))
                    s = BeautifyUpdate(s);
                else if (s.StartsWith("DELETE", StringComparison.OrdinalIgnoreCase))
                    s = BeautifySelDel("DELETE", s, "");
                sqlText.Text = sqlText.Text.Remove(startPosition, textLength);
                sqlText.Text = sqlText.Text.Insert(startPosition, s);
                sqlText.SelectionStart = startPosition;
                sqlText.SelectionLength = 0;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        private void btnCreateSelect_Click(object sender, EventArgs e)
        {
            if (dbTreeView.SelectedNode == null || n7.nvl(dbTreeView.SelectedNode.Tag) == "")
            {
                MessageBox.Show("Please select an object first.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            DataTable dt = new DataTable();
            n7.DataTableFill(dt, "select * from " + dbTreeView.SelectedNode.Text + " where 1=2", null, true);

            string q = "SELECT ";
            for (int i = 0; i < dt.Columns.Count; i++)
            {
                q += dt.Columns[i].ColumnName;
                if (i < dt.Columns.Count - 1)
                    q += ", ";
            }
            q += " FROM " + dbTreeView.SelectedNode.Text;

            if (dt.PrimaryKey.Length > 0)
            {
                q += " WHERE ";
                for (int i = 0; i < dt.PrimaryKey.Length; i++)
                {
                    q += dt.PrimaryKey[i].ColumnName + " = " + n7.ProviderParam + dt.PrimaryKey[i].ColumnName;
                    if (i < dt.PrimaryKey.Length - 1)
                        q += "  AND ";
                }
            }
            else
                q += " WHERE <conditions>";

            sqlText.Text = BeautifySelDel("SELECT", q, "");
        }
        private void btnCreateInsert_Click(object sender, EventArgs e)
        {
            if (dbTreeView.SelectedNode == null || n7.nvl(dbTreeView.SelectedNode.Tag) == "")
            {
                MessageBox.Show("Please select an object first.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            DataTable dt = new DataTable();
            n7.DataTableFill(dt, "select * from " + dbTreeView.SelectedNode.Text + " where 1=2");

            string q = "INSERT INTO " + dbTreeView.SelectedNode.Text + " (";
            for (int i = 0; i < dt.Columns.Count; i++)
                q += dt.Columns[i].ColumnName + ", ";
            q = n7.RTrunc(q, 2) + ")\r\nVALUES (";
            for (int i = 0; i < dt.Columns.Count; i++)
                q += n7.ProviderParam + dt.Columns[i].ColumnName + ", ";
            q = n7.RTrunc(q, 2) + ")";

            // splitting lines
            int k = 0;
            sqlText.Text = "";
            bool apex = false;

            for (int i = 0; i < q.Length; i++)
            {
                if (i < q.Length - 1 && q.Substring(i, 2) == "\r\n")
                    k = 0;
                if (q.Substring(i, 1) == "'" && apex == false)
                    apex = true;
                if (q.Substring(i, 1) == "'" && apex == true)
                    apex = false;
                if (k > 70 && apex == false && q.Substring(i, 1) == " ")
                {
                    sqlText.Text += "\r\n";
                    k = 0;
                }
                sqlText.Text += q.Substring(i, 1);
                k++;
            }
        }
        private void btnCreateUpdate_Click(object sender, EventArgs e)
        {
            if (dbTreeView.SelectedNode == null || n7.nvl(dbTreeView.SelectedNode.Tag) == "")
            {
                MessageBox.Show("Please select an object first.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            DataTable dt = new DataTable();
            n7.DataTableFill(dt, "select * from " + dbTreeView.SelectedNode.Text + " where 1=2", null, true);

            string q = "UPDATE " + dbTreeView.SelectedNode.Text + "\r\n   SET ";
            for (int i = 0; i < dt.Columns.Count; i++)
            {
                if (i > 0)
                    q += "       ";
                q += dt.Columns[i].ColumnName + " = " + n7.ProviderParam + "new_" + dt.Columns[i].ColumnName + ", \r\n";
            }
            q = n7.RTrunc(q, 4) + "\r\n WHERE ";
            if (dt.PrimaryKey.Length > 0)
            {
                for (int i = 0; i < dt.PrimaryKey.Length; i++)
                {
                    q += dt.PrimaryKey[i].ColumnName + " = " + n7.ProviderParam + dt.PrimaryKey[i].ColumnName + "\r\n";
                    if (i < dt.PrimaryKey.Length - 1)
                        q += "   AND ";
                }
            }
            else
                q += "<conditions>";
            sqlText.Text = q;
        }
        private void btnCreateDelete_Click(object sender, EventArgs e)
        {
            if (dbTreeView.SelectedNode == null || n7.nvl(dbTreeView.SelectedNode.Tag) == "")
            {
                MessageBox.Show("Please select an object first.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }

            DataTable dt = new DataTable();
            n7.DataTableFill(dt, "select * from " + dbTreeView.SelectedNode.Text + " where 1=2", null, true);

            string q = "DELETE " + dbTreeView.SelectedNode.Text + "\r\n WHERE ";
            if (dt.PrimaryKey.Length > 0)
            {
                for (int i = 0; i < dt.PrimaryKey.Length; i++)
                {
                    q += dt.PrimaryKey[i].ColumnName + " = " + n7.ProviderParam + dt.PrimaryKey[i].ColumnName + "\r\n";
                    if (i < dt.PrimaryKey.Length - 1)
                        q += "   AND ";
                }
            }
            else
                q += "<conditions>";
            sqlText.Text = q;
        }
        private void btnCreateTable_Click(object sender, EventArgs e)
        {
            TreeNode tn = dbTreeView.SelectedNode;
            if (tn == null || tn.Tag == null || tn.Tag.ToString() != "T") return;

            DataTable dt = new DataTable();
            string q;
            string s = "CREATE TABLE " + tn.Text + "\r\n(\r\n";

            if (n7.ProviderType == "SQL")
            {
                string t = tn.Text;
                if (t.IndexOf('.') > 0)
                    t = t.Split('.')[1];

                q = "SELECT name column_name, (select top 1 lower(name) from systypes where xtype=syscolumns.xtype) data_type, length data_length, xprec data_precision, xscale data_scale, (case isnullable when 0 then 'NOT NULL' end) nullable"
                + "    FROM syscolumns"
                + "   WHERE id = (select id from sysobjects where name='" + t + "') "
                + "   ORDER BY colorder";
                n7.DataTableFill(dt, q);

                for (int j = 0; j < dt.Rows.Count; j++)
                {
                    if (j > 0) s += ", \r\n";
                    if (dt.Rows[j]["data_type"].ToString() == "char" || dt.Rows[j]["data_type"].ToString() == "varchar" || dt.Rows[j]["data_type"].ToString() == "nvarchar")
                        s += dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + "(" + dt.Rows[j]["data_length"].ToString() + ") " + n7.nvl(dt.Rows[j]["nullable"]);
                    else if (dt.Rows[j]["data_type"].ToString() == "numeric" && n7.nvl(dt.Rows[j]["data_precision"]) != "")
                        s += dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + "(" + n7.nvl(dt.Rows[j]["data_precision"]) + "," + n7.nvl(dt.Rows[j]["data_scale"]) + ") " + n7.nvl(dt.Rows[j]["nullable"]);
                    else
                        s += dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + " " + n7.nvl(dt.Rows[j]["nullable"]);
                }
                s += "\r\n)";
            }
            else if (n7.ProviderType == "ORA")
            {
                if (showAllObj == 1)
                    q = "SELECT column_name, lower(data_type) data_type, data_length, data_precision, data_scale, decode(nullable,'N','not null',null) nullable"
                    + "    FROM all_tab_columns"
                    + "   WHERE owner = '" + tn.Text.Split('.')[0] + "'"
                    + "     AND table_name = '" + tn.Text.Split('.')[1] + "'"
                    + " ORDER BY column_id";
                else
                    q = "SELECT column_name, lower(data_type) data_type, data_length, data_precision, data_scale, decode(nullable,'N','not null',null) nullable"
                    + "    FROM user_tab_columns"
                    + "   WHERE table_name = '" + tn.Text + "'"
                    + "   ORDER BY column_id";
                n7.DataTableFill(dt, q);

                for (int j = 0; j < dt.Rows.Count; j++)
                {
                    if (j > 0) s += ", \r\n";
                    if (dt.Rows[j]["data_type"].ToString() == "char" || dt.Rows[j]["data_type"].ToString() == "varchar" || dt.Rows[j]["data_type"].ToString() == "varchar2")
                        s += dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + "(" + dt.Rows[j]["data_length"].ToString() + ") " + n7.nvl(dt.Rows[j]["nullable"]);
                    else if (dt.Rows[j]["data_type"].ToString() == "number" && n7.nvl(dt.Rows[j]["data_precision"]) != "")
                        s += dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + "(" + n7.nvl(dt.Rows[j]["data_precision"]) + "," + n7.nvl(dt.Rows[j]["data_scale"]) + ") " + n7.nvl(dt.Rows[j]["nullable"]);
                    else
                        s += dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + " " + n7.nvl(dt.Rows[j]["nullable"]);
                }
                s += "\r\n)";
            }
            else if (n7.ProviderType == "MYSQL")
            {
                string[] r = new string[3];
                r[0] = null;
                if (tn.Text.IndexOf('.') > 0)
                {
                    r[1] = tn.Text.Split('.')[0];
                    r[2] = tn.Text.Split('.')[1];
                }
                else
                {
                    r[1] = null;
                    r[2] = tn.Text;
                }
                DbConnection cn = n7.CreateConnection();
                cn.Open();
                dt = cn.GetSchema("Columns", r);
                cn.Close();

                for (int j = 0; j < dt.Rows.Count; j++)
                {
                    if (j > 0) s += ", \r\n";
                    s += dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["column_type"].ToString();
                }
                s += "\r\n)";
            }

            dt.Clear();
            n7.DataTableFill(dt, "select * from " + tn.Text + " where 1=2", null, true);
            if (dt.PrimaryKey.Length > 0)
            {
                s += "\r\n/\r\nALTER TABLE " + tn.Text + " ADD PRIMARY KEY (";
                for (int j = 0; j < dt.PrimaryKey.Length; j++)
                {
                    if (j > 0) s += ",";
                    s += dt.PrimaryKey[j].ColumnName;
                }
                s += ")\r\n/";
            }

            sqlText.Text = s;
        }

        private void btnNewQuery_Click(object sender, EventArgs e)
        {
            if (sqlTextModified && MessageBox.Show("Command has been modified, do you want to continue?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                return;
            sqlText.Text = "";
            sqlTextModified = false;
            bindingSource.DataSource = null;
            SetStatusText("");
        }
        private void btnOpenQuery_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "Sql script|*.sql|Text files|*.txt|All files|*";
            openFileDialog1.FileName = "";
            openFileDialog1.ShowDialog();
            string fileName = openFileDialog1.FileName;
            if (fileName == "") return;

            try
            {
                StreamReader f = new StreamReader(fileName);
                sqlText.Text = f.ReadToEnd();
                f.Close();

                sqlTextModified = false;
                sqlName = fileName;
            }
            catch
            {
            }
            SetStatusText("");
        }
        private void btnSaveQuery_Click(object sender, EventArgs e)
        {
            if (sqlText.Text != "")
            {
                SaveFileDialog saveFileDialog1 = new SaveFileDialog();
                saveFileDialog1.Filter = "Sql script|*.sql|Text files|*.txt";
                saveFileDialog1.ShowDialog();
                string fileName = saveFileDialog1.FileName;
                if (fileName == "") return;

                n7.FileWrite(fileName, sqlText.Text, false);
                sqlName = fileName;
            }
            SetStatusText("");
        }

        private void btnExecuteMenu_ButtonClick(object sender, EventArgs e)
        {
            btnExecuteQuery_Click(sender, e);
        }
        private void btnExecuteQuery_Click(object sender, EventArgs e)
        {
            if (n7.ConnectionString == "")
            {
                MessageBox.Show("Not connected to a database", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return;
            }

            int i;
            string q, q1;
            if (sqlText.SelectedText != "")
                q = sqlText.SelectedText;
            else if (sqlText.Text.IndexOf("\r\n/") > 0)
            {
                int actualPosition = sqlText.SelectionStart;
                int startPosition = sqlText.Text.LastIndexOf("\r\n/", actualPosition);
                if (startPosition < 0) startPosition = 0;
                int endPosition = sqlText.Text.IndexOf("\r\n/", actualPosition);
                if (endPosition < 0) endPosition = sqlText.Text.Length;

                q = sqlText.Text.Substring(startPosition, endPosition - startPosition);
                q = q.Replace("\r\n/\r\n", "");
                q = q.Replace("\r\n/", "");
                q = q.Replace("/\r\n", "");
            }
            else if (sqlText.Text.IndexOf(";\r\n") > 0)
            {
                int actualPosition = sqlText.SelectionStart;
                int startPosition = sqlText.Text.LastIndexOf(";\r\n", actualPosition);
                if (startPosition < 0) startPosition = 0;
                int endPosition = sqlText.Text.IndexOf(";\r\n", actualPosition);
                if (endPosition < 0) endPosition = sqlText.Text.Length;

                q = sqlText.Text.Substring(startPosition, endPosition - startPosition);
                q = q.Replace(";\r\n", "");
            }
            else
                q = sqlText.Text;

            q1 = q;
            q = QueryCleanUp(q, false);
            if (q == "") return;

            // parameters request
            string fieldsList = "";
            bool isParam = false;
            bool apex = false;
            for (i = 0; i < sqlText.Text.Length; i++)
            {
                if (sqlText.Text.Substring(i, 1) == "'" && apex == false)
                    apex = true;
                else if (sqlText.Text.Substring(i, 1) == "'" && apex == true)
                    apex = false;
                if (!apex)
                {
                    if (sqlText.Text.Substring(i, 1) == n7.ProviderParam)
                    {
                        isParam = true;
                        i++;
                    }
                    if (isParam && sqlText.Text.Substring(i, 1) != n7.StringFilter(sqlText.Text.Substring(i, 1), netlib7.StringType.CharsAndNumbers, "_"))
                    {
                        isParam = false;
                        fieldsList += ";";
                    }
                    if (isParam)
                    {
                        fieldsList += sqlText.Text.Substring(i, 1);
                    }
                }
            }

            if (fieldsList == "")
                parameters = null;
            else
            {
                if (fieldsList.EndsWith(";"))
                    fieldsList = n7.RTrunc(fieldsList, 1);

                frmParams f = new frmParams();
                f.parameters = parameters;
                f.fields = fieldsList;
                f.ShowDialog();
                parameters = f.parameters;
                if (parameters == null)
                {
                    SetStatusText("");
                    return;
                }
            }

            // query execution
            bindingSource.DataSource = null;
            Cursor.Current = Cursors.WaitCursor;

            string queryType;
            if (q.ToUpper().StartsWith("SELECT") && n7.ProviderType == "SQL" && n7.SubString(q.ToUpper(), 0, q.IndexOf(" FROM ")).IndexOf(" INTO ") > 0)
                queryType = "DDL";
            else if (q.ToUpper().StartsWith("SELECT"))
                queryType = "SELECT";
            else if (q.ToUpper().StartsWith("INSERT") ||
                     q.ToUpper().StartsWith("UPDATE") ||
                     q.ToUpper().StartsWith("DELETE"))
                queryType = "DML";
            else
                queryType = "DDL";

            if (queryType == "SELECT")
            {
                DbCommand cm = null;
                DbDataReader rd = null;
                DataTable dt = new DataTable();

                SetStatusText("Executing query...");
                try
                {
                    // retrieve table schema ...
                    if (parameters != null)
                    {
                        DbParameter[] p = new DbParameter[parameters.Length];
                        for (int j = 0; j < parameters.Length; j++)
                        {
                            p[j] = n7.CreateParameter();
                            p[j].ParameterName = parameters[j].ParameterName;
                            p[j].Value = parameters[j].Value;
                        }
                        n7.DataTableFill(dt, n7.AddWhere(q, "1=2"), p, false);
                    }
                    else
                        n7.DataTableFill(dt, n7.AddWhere(q, "1=2"), null, false);
                    if (n7.ErrorMessage != "")
                        throw new Exception(n7.ErrorMessage);

                    dt.Clear();
                    bindingSource.DataSource = dt;
                    dataGrid.DataSource = bindingNavigator.BindingSource;

                    // retrieve data ...
                    cm = n7.CreateCommand(q);
                    if (parameters != null)
                        cm.Parameters.AddRange(parameters);
                    rd = n7.CreateDataReader(cm);
                    if (rd == null)
                        throw new Exception(n7.ErrorMessage);

                    btnStop.Enabled = true;
                    while (rd.Read() && btnStop.Enabled)
                    {
                        DataRow rw = dt.NewRow();
                        for (int k = 0; k < dt.Columns.Count; k++)
                            rw[k] = rd[k];
                        dt.Rows.Add(rw);

                        Cursor.Current = Cursors.AppStarting;
                        if (Convert.ToInt32(dt.Rows.Count / 130) == Convert.ToDouble(dt.Rows.Count / 130.00))
                            SetStatusText(dt.Rows.Count.ToString() + " record(s) retrieved");
                    }

                    AddToHistory(q1);
                    i = dt.Rows.Count;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    i = -1;
                }
                finally
                {
                    if (cm != null) cm.Cancel();
                    if (rd != null && !rd.IsClosed) rd.Close();
                    dt.AcceptChanges();
                    dt.BeginLoadData();
                    btnStop.Enabled = false;
                }
                if (i >= 0)
                    SetStatusText(i.ToString() + " record(s)");
                else
                    SetStatusText("");
            }
            else if (queryType == "DML")
            {
                DbConnection cn = null;
                DbTransaction tr = null;
                DbCommand cm = null;

                if (queryOnlyMode)
                {
                    MessageBox.Show("You are not authorized to run this query", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    return;
                }

                bool useTran = true;
                SetStatusText("Working...");
                try
                {
                    cn = n7.CreateConnection();
                    cn.Open();
                    try
                    {
                        tr = cn.BeginTransaction();
                    }
                    catch
                    {
                        useTran = false;
                    }
                    cm = n7.CreateCommand(cn, q);
                    cm.CommandTimeout = 0;
                    if (useTran) cm.Transaction = tr;
                    if (parameters != null)
                        cm.Parameters.AddRange(parameters);
                    i = cm.ExecuteNonQuery();
                    AddToHistory(q1);

                    if (useTran)
                    {
                        if (MessageBox.Show(i.ToString() + " record(s) affected\r\nDo you want to commit?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                            tr.Commit();
                        else
                        {
                            tr.Rollback();
                            i = -1;
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    if (useTran) tr.Rollback();
                    i = -1;
                }
                finally
                {
                    if (cn.State == ConnectionState.Open)
                        cn.Close();
                }
                if (i >= 0)
                    SetStatusText(i.ToString() + " record(s) affected");
                else
                    SetStatusText("");
            }
            else
            {
                if (queryOnlyMode)
                {
                    MessageBox.Show("You are not authorized to run this query or is not a valid query.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    return;
                }

                SetStatusText("Working...");
                i = n7.ExecNonQuery(q, parameters);
                if (i < 0 && n7.ErrorMessage != "")
                {
                    MessageBox.Show(n7.ErrorMessage, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    SetStatusText("");
                }
                else
                {
                    SetStatusText("Command completed successfully");
                    AddToHistory(q1);
                }
            }

            Cursor.Current = null;
        }
        private void btnExecuteScript_Click(object sender, EventArgs e)
        {
            string r = "";
            if (n7.ConnectionString == "")
            {
                MessageBox.Show("Not connected to a database", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return;
            }

            string s;
            if (sqlText.SelectedText != "")
                s = sqlText.SelectedText;
            else
                s = sqlText.Text;
            s += "\r\n";

            ArrayList v = new ArrayList();
            bool apex = false;
            int j = 0;
            for (int i = 0; i < s.Length; i++)
            {
                if (s.Substring(i, 1) == "'" && apex == false)
                    apex = true;
                else if (s.Substring(i, 1) == "'" && apex == true)
                    apex = false;
                if (apex == false)
                {
                    if (i + 2 > s.Length)
                    {
                        v.Add(s.Substring(j));
                    }
                    else if (s.Substring(i, 1) == ";" && s.Substring(i + 1, 2) == "\r\n")
                    {
                        v.Add(s.Substring(j, i - j));
                        j = i + 3;
                    }
                    else if (s.Substring(i, 1) == "/" && s.Substring(i + 1, 2) == "\r\n" && s.Substring(i - 2, 2) == "\r\n")
                    {
                        v.Add(s.Substring(j, i - j));
                        j = i + 3;
                    }
                }
            }

            if (v.Count == 0)
            {
                MessageBox.Show("No script to execute", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return;
            }

            Cursor.Current = Cursors.WaitCursor;
            SetStatusText("Working");

            DbConnection cn = null;
            DbTransaction tr = null;
            DbCommand cm = null;

            bool useTran = true;
            cn = n7.CreateConnection();
            cn.Open();
            try
            {
                tr = cn.BeginTransaction();
            }
            catch
            {
                useTran = false;
            }
            cm = n7.CreateCommand(cn);
            cm.CommandTimeout = 0;
            cm.CommandType = CommandType.Text;
            if (useTran) cm.Transaction = tr;

            for (int k = 0; k < v.Count; k++)
            {
                cm.CommandText = QueryCleanUp(v[k].ToString(), true);
                if (cm.CommandText != "")
                {
                    if (cm.CommandText.Length > 70)
                        r += cm.CommandText.Substring(0, 70) + " ...";
                    else
                        r += cm.CommandText;
                    if (!r.EndsWith("\r\n")) r += "\r\n";
                    r += "\r\n";

                    try
                    {
                        if (cm.CommandText.StartsWith("SELECT", StringComparison.OrdinalIgnoreCase))
                        {
                            DbDataAdapter da = n7.CreateDataAdapter(cm);
                            DataTable dt = new DataTable();
                            da.Fill(dt);

                            int[] l = new int[dt.Columns.Count];
                            for (int jc = 0; jc < dt.Columns.Count; jc++)
                            {
                                l[jc] = 0;
                                if (dt.Rows.Count > 0)
                                    l[jc] = dt.Rows[0][jc].ToString().Length;
                                if (dt.Columns[jc].ColumnName.Length > l[jc])
                                    l[jc] = dt.Columns[jc].ColumnName.Length;
                            }
                            for (int jc = 0; jc < dt.Columns.Count; jc++)
                                r += dt.Columns[jc].ColumnName.PadRight(l[jc], ' ') + " ";
                            r += "\r\n";
                            for (int jc = 0; jc < dt.Columns.Count; jc++)
                                r += Space(l[jc]).Replace(' ', '_') + " ";
                            r += "\r\n";
                            for (int jr = 0; jr < dt.Rows.Count; jr++)
                            {
                                for (int jc = 0; jc < dt.Columns.Count; jc++)
                                    r += n7.SubString(dt.Rows[jr][jc].ToString(), 0, l[jc]).PadRight(l[jc], ' ') + " ";
                                r += "\r\n";
                            }
                            r += "\r\n" + dt.Rows.Count.ToString() + " record(s)\r\n";
                        }
                        else
                        {
                            int i = cm.ExecuteNonQuery();
                            if (cm.CommandText.StartsWith("INSERT", StringComparison.OrdinalIgnoreCase) ||
                                cm.CommandText.StartsWith("UPDATE", StringComparison.OrdinalIgnoreCase) ||
                                cm.CommandText.StartsWith("DELETE", StringComparison.OrdinalIgnoreCase))
                                r += i.ToString() + " record(s) affected\r\n";
                            else
                                r += "Command completed succesfully\r\n";
                        }
                    }
                    catch (Exception ex)
                    {
                        r += ex.Message + "\r\n";
                    }
                    r += "-".PadRight(130, '-');
                    r += "\r\n";
                }
            }

            frmEdit t = new frmEdit();
            t.textValue = r;
            t.readOnly = true;
            t.ShowDialog();

            if (useTran && tr.Connection != null)
            {
                if (MessageBox.Show("Do you want to commit?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                    tr.Commit();
                else
                    tr.Rollback();
            }
            cn.Close();

            Cursor.Current = null;
            SetStatusText("");

            AddToHistory(s);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            btnStop.Enabled = false;
            cancel = true;
            n7.Cancel = true;
        }

        private void btnCommitMenu_ButtonClick(object sender, EventArgs e)
        {
            btnCommit_Click(sender, e);
        }
        private void btnCommit_Click(object sender, EventArgs e)
        {
            if (queryOnlyMode) return;

            if (n7.ConnectionString == "")
            {
                MessageBox.Show("Not connected to a database", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return;
            }

            if (dataGrid.Rows.Count == 0)
                return;

            if (MessageBox.Show("Are you sure you want to save all pending changes?", Application.ProductName, MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.No) return;

            DataTable dt = new DataTable();
            dt = (DataTable)bindingSource.DataSource;

            if (n7.DataTableUpdate(dt, lastSql) == false)
                MessageBox.Show("There are rows with error that cannot be saved.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);

            dataGrid.Refresh();
        }
        private void btnShowError_Click(object sender, EventArgs e)
        {
            if (dataGrid.CurrentRow == null)
                return;

            string err = "";
            try
            {
                err = ((DataTable)bindingSource.DataSource).Rows[dataGrid.CurrentRow.Index].RowError;
            }
            catch
            {
                err = "";
            }
            if (err == "")
                err = n7.ErrorMessage;
            if (err != "")
                MessageBox.Show(err, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
        }

        private void btnHelp_Click(object sender, EventArgs e)
        {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(freeSQL.Properties.Resources));
            string helpText = resources.GetString("helpText");
            helpText = helpText.Replace("$version", Application.ProductVersion);
            helpText = helpText.Replace("$date", System.IO.File.GetCreationTime(Application.ExecutablePath).ToString("D"));
            n7.FileWrite(Environment.GetEnvironmentVariable("TEMP") + "\\freeSQL.htm", helpText, false);
            System.Diagnostics.Process p = new System.Diagnostics.Process();
            p.StartInfo.FileName = Environment.GetEnvironmentVariable("TEMP") + "\\freeSQL.htm";
            p.StartInfo.UseShellExecute = true;
            p.Start();
        }

        private void bindingNavigatorKeyItem_Click(object sender, EventArgs e)
        {
            if (dataGrid.Rows.Count == 0) return;

            string s = "";
            for (int i = dataGrid.SelectedCells.Count - 1; i >= 0; i--)
                s += dataGrid.Columns[dataGrid.SelectedCells[i].ColumnIndex].Name + ",";
            s = n7.RTrunc(s, 1);

            n7.DataTableSetPKey((DataTable)bindingSource.DataSource, s);
            MessageBox.Show("Temporary primary key is now " + s, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
        private void bindingNavigatorDupRecItem_Click(object sender, EventArgs e)
        {
            if (dataGrid.Rows.Count == 0) return;
            int c = dataGrid.CurrentRow.Index;

            DataTable dt = (DataTable)bindingSource.DataSource;
            DataRow rw = dt.NewRow();
            dt.Rows.InsertAt(rw, c + 1);
            for (int i = 0; i < dt.Columns.Count; i++)
                dt.Rows[c + 1][i] = dt.Rows[c][i];

            dataGrid.CurrentCell = dataGrid.Rows[c + 1].Cells[0];
            dataGrid.BeginEdit(true);
        }
        private void btnExport_Click(object sender, EventArgs e)
        {
            if (n7.ConnectionString == "")
            {
                MessageBox.Show("Not connected to a database", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return;
            }

            if (dataGrid.Rows.Count == 0)
                return;

            string tableName = "";
            if (dbTreeView.SelectedNode!=null && n7.IsIn(n7.nvl(dbTreeView.SelectedNode.Tag), "T,V,S"))
                tableName = dbTreeView.SelectedNode.Text;
            else
            {
                int p1, p2;
                string s = QueryCleanUp(sqlText.Text, true);
                p1 = s.IndexOf("FROM ", StringComparison.OrdinalIgnoreCase);
                p1 += 5;
                if (p1 > 0)
                {
                    p2 = s.IndexOf(" ", p1);
                    if (p2 > 0)
                        tableName = s.Substring(p1, p2 - p1);
                }
            }

            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "XML format|*.xml|"
                                   + "Excel compatible format|*.csv|"
                                   + "Text file delimited by tab|*.txt|"
                                   + "Text file without delimiter|*.dat|"
                                   + "Insert statements|*.sql";
            saveFileDialog1.FileName = tableName;
            saveFileDialog1.ShowDialog();
            string fileName = saveFileDialog1.FileName;
            if (fileName == "") return;

            string ext = System.IO.Path.GetExtension(fileName).ToLower();
            if (ext == ".xml")
            {
                DataTable dt = (DataTable)bindingSource.DataSource;
                dt.TableName = "xmlTable";
                dt.WriteXml(fileName, XmlWriteMode.WriteSchema);
            }
            else if (ext == ".sql")
            {
                StreamWriter f = new StreamWriter(fileName);

                DataTable dt = (DataTable)bindingSource.DataSource;
                string s = "INSERT INTO ";
                s += tableName;
                s += " (";
                for (int i = 0; i < dt.Columns.Count; i++)
                {
                    if (i > 0) s += ", ";
                    s += dt.Columns[i].ColumnName;
                }
                s += ") VALUES (";

                string v;
                for (int j = 0; j < dt.Rows.Count; j++)
                {
                    v = "";
                    for (int i = 0; i < dt.Columns.Count; i++)
                    {
                        if (i > 0) v += ", ";
                        if (dt.Rows[j][i] == DBNull.Value)
                            v += "null";
                        else if (dt.Columns[i].DataType == Type.GetType("System.String"))
                            v += "'" + dt.Rows[j][i].ToString().Replace("'", "''") + "'";
                        else if (dt.Columns[i].DataType == Type.GetType("System.DateTime"))
                        {
                            if (n7.ProviderType == "ORA")
                                v += "to_date('" + Convert.ToDateTime(dt.Rows[j][i]).ToString("yyyy-MM-dd HH\\:mm\\:ss") + "','yyyy-mm-dd hh24:mi:ss')";
                            else if (n7.ProviderType == "SQL")
                                v += "Convert(DateTime,'" + Convert.ToDateTime(dt.Rows[j][i]).ToString("yyyy-MM-dd HH\\:mm\\:ss") + "',120)";
                            else if (n7.ProviderType == "ACC")
                                v += "#" + Convert.ToDateTime(dt.Rows[j][i]).ToString("dd/MM/yyyy HH:mm:ss") + "#";
                        }
                        else if (dt.Columns[i].DataType == Type.GetType("System.Decimal"))
                            v += dt.Rows[j][i].ToString().Replace(",", ".");
                        else
                        {
                            if (dt.Rows[j][i].ToString().StartsWith("System"))
                                v += "null";
                            else
                                v += dt.Rows[j][i].ToString();
                        }
                    }
                    v += ")";
                    f.WriteLine(s + v + ";");
                }

                f.Close();
            }
            else
            {
                string writer, fieldEncloser, fieldDelimiter, data;
                if (ext == ".csv")
                {
                    fieldEncloser = "\"";
                    fieldDelimiter = ";";
                }
                else if (ext == ".txt")
                {
                    fieldEncloser = "";
                    fieldDelimiter = Convert.ToChar(9).ToString();
                }
                else
                {
                    fieldEncloser = "";
                    fieldDelimiter = "";
                }

                StreamWriter f = new StreamWriter(fileName);
                writer = "";

                // heading
                if (fieldDelimiter != "")
                {
                    for (int h = 0; h < dataGrid.ColumnCount; h++)
                        writer += fieldEncloser + dataGrid.Columns[h].HeaderText + fieldEncloser + fieldDelimiter;
                    f.WriteLine(n7.RTrunc(writer, 1));
                }

                // data
                for (int i = 0; i < dataGrid.Rows.Count; i++)
                {
                    writer = "";
                    for (int j = 0; j < dataGrid.Rows[i].Cells.Count; j++)
                    {
                        data = n7.nvl(dataGrid.Rows[i].Cells[j].Value);
                        if (ext == ".csv")
                            data = data.Replace("\"", "\"\"");
                        writer += fieldEncloser + data + fieldEncloser + fieldDelimiter;
                    }
                    if (fieldDelimiter != "")
                        writer = n7.RTrunc(writer, 1);
                    f.WriteLine(writer);
                }
                f.Close();
            }
        }
        private void btnEdit_Click(object sender, EventArgs e)
        {
            if (dataGrid.CurrentCell != null)
                EditCell(dataGrid.CurrentCell.RowIndex, dataGrid.CurrentCell.ColumnIndex);
        }
        private void btnPrint_Click(object sender, EventArgs e)
        {
            if (dataGrid.Rows.Count == 0)
                return;

            SetStatusText("Working... (press ESC to abort)");

            int totalWidth = 0;
            for (int i = 0; i < dataGrid.ColumnCount; i++)
                totalWidth += dataGrid.Columns[i].Width;

            string ColumnsNames = "", ColumnsWidth = "";
            int width;

            int tableTotalWidth = totalWidth / 10;
            if (tableTotalWidth > 100)
                tableTotalWidth = 100;

            for (int i = 0; i < dataGrid.ColumnCount; i++)
            {
                ColumnsNames += dataGrid.Columns[i].HeaderText + ";";
                width = dataGrid.Columns[i].Width * tableTotalWidth / totalWidth;
                if (width <= 0) width = 1;
                if (dataGrid.Columns[i].ValueType == Type.GetType("System.DateTime") && width < 10)
                    ColumnsWidth += width.ToString() + "{0:dd/MM/yyyy}";
                else if (dataGrid.Columns[i].ValueType == Type.GetType("System.Decimal"))
                    ColumnsWidth += "*" + width.ToString();
                else
                    ColumnsWidth += width.ToString();
                ColumnsWidth += ";";
            }
            ColumnsWidth = n7.RTrunc(ColumnsWidth, 1);

            DataTable dt = (DataTable)bindingSource.DataSource;

            ToolStripProgressBar p = (ToolStripProgressBar)statusBar.Items["pBar"];
            p.Value = 0;
            p.Maximum = dt.Rows.Count;
            p.Visible = true;
            Application.DoEvents();

            string h = "</h2>\n<p align=center>\n<table><tr><td width=25%> </td><td width=50%>" + lastSql.Replace("\r\n", "<br>") + "</td><td width=25%> </td></tr></table>\n</p>\n<h2>";
            n7.DataTableToHtml(dt, sqlName, h, ColumnsNames, ColumnsWidth, "", "", true, true);

            p.Visible = false;
            SetStatusText("");
            this.Cursor = null;
        }
        private void btnSizeGrid_Click(object sender, EventArgs e)
        {
            btnSizeGrid.Checked = (!btnSizeGrid.Checked);
            if (btnSizeGrid.Checked)
                dataGrid.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.DisplayedCellsExceptHeader;
            else
                dataGrid.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.None;
        }
        private void btnEditOnEnter_Click(object sender, EventArgs e)
        {
            if (btnEditOnEnter.Checked)
            {
                dataGrid.EditMode = DataGridViewEditMode.EditOnKeystroke;
                btnEditOnEnter.Checked = false;
            }
            else
            {
                dataGrid.EditMode = DataGridViewEditMode.EditOnEnter;
                btnEditOnEnter.Checked = true;
            }
        }

        private void dataGrid_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {

        }

        private void dbTreeView_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            TreeNode tn = e.Node;
            if (tn.FirstNode == null || tn.FirstNode.Text != ".") return;
            tn.FirstNode.Remove();

            if (n7.ProviderName == "") return;

            dbTreeView.Cursor = Cursors.AppStarting;
            SetStatusText("Working...");

            string[] r = null;

            if (tn.Tag == null || tn.Tag.ToString() == "")
            {
                DataTable dt = new DataTable();
                TreeNode tn1;
                string oType = n7.RTrunc(tn.Text, 1).ToUpper();

                if (n7.ProviderType == "SQL")
                {
                    string xType = "";
                    if (oType == "TABLE")
                        xType = "U";
                    else if (oType == "VIEW")
                        xType = "V";
                    else if (oType == "SYNONYM")
                        xType = "SN";
                    n7.DataTableFill(dt, "select null as table_catalog, 'dbo' as table_schema, name as table_name, '" + oType + "' as table_type from sysobjects where xtype='" + xType + "' order by 3");
                }
                else if (n7.ProviderType == "ORA")
                {
                    if (showAllObj == 0)
                        n7.DataTableFill(dt, "select null as table_catalog, '" + n7.GetConnectionProperty("user id") + "' as table_schema, object_name as table_name, '" + oType + "' as table_type from user_objects where object_type='" + oType + "' order by 3");
                    else
                        n7.DataTableFill(dt, "select null as table_catalog, decode(owner,'PUBLIC','',owner) as table_schema, object_name as table_name, '" + oType + "' as table_type from all_objects where object_type='" + oType + "' and owner not like 'SYS%' and instr(object_name,'/')=0  order by 2,3");
                }
                else if (n7.ProviderType == "MYSQL")
                {
                    r = new string[2];
                    r[0] = null;

                    if (oType == "TABLE")
                        oType = "BASE TABLE";
                    if (showAllObj == 0)
                        r[1] = n7.GetConnectionProperty("database");
                    else
                        r[1] = null;
                    dt = null;
                }
                else
                    dt = null;

                if (dt == null)
                {
                    DbConnection cn = n7.CreateConnection();
                    cn.Open();
                    dt = cn.GetSchema("Tables", r);
                    cn.Close();
                }

                cancel = false;
                SetStatusText("Loading " + tn.Text.ToLower() + " (ESC abort)");
                ToolStripProgressBar p = (ToolStripProgressBar)statusBar.Items["pBar"];
                p.Value = 0;
                p.Maximum = dt.Rows.Count;
                p.Visible = true;
                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    if (dt.Rows[i]["table_type"].ToString() == oType)
                    {
                        if (showAllObj == 1 && dt.Rows[i]["table_schema"].ToString() != "")
                            tn1 = tn.Nodes.Add(dt.Rows[i]["table_schema"].ToString() + "." + dt.Rows[i]["table_name"].ToString());
                        else
                            tn1 = tn.Nodes.Add(dt.Rows[i]["table_name"].ToString());
                        tn1.Nodes.Add(".");
                        tn1.Tag = tn.Text.Substring(0, 1);
                    }
                    p.Value++;
                    dbTreeView.Cursor = Cursors.AppStarting;
                    Application.DoEvents();
                    if (cancel) break;
                }
                p.Visible = false;
            }
            else
            {
                DataTable dt = new DataTable();
                TreeNode tn1;
                string q, s;

                if ((tn.Tag.ToString() == "T" || tn.Tag.ToString() == "V") && n7.ProviderType == "SQL")
                {
                    string t = tn.Text;
                    if (t.IndexOf('.') > 0)
                        t = t.Split('.')[1];

                    q = "SELECT name column_name, (select top 1 lower(name) from systypes where xtype=syscolumns.xtype) data_type, length data_length, xprec data_precision, xscale data_scale, (case isnullable when 0 then 'NOT NULL' end) nullable"
                    + "    FROM syscolumns"
                    + "   WHERE id = (select id from sysobjects where name='" + t + "') "
                    + "   ORDER BY colorder";
                    n7.DataTableFill(dt, q);
                    for (int j = 0; j < dt.Rows.Count; j++)
                    {
                        if (dt.Rows[j]["data_type"].ToString() == "char" || dt.Rows[j]["data_type"].ToString() == "varchar" || dt.Rows[j]["data_type"].ToString() == "nvarchar")
                            s = dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + "(" + dt.Rows[j]["data_length"].ToString() + ") " + n7.nvl(dt.Rows[j]["nullable"]);
                        else if (dt.Rows[j]["data_type"].ToString() == "numeric" && n7.nvl(dt.Rows[j]["data_precision"]) != "")
                            s = dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + "(" + n7.nvl(dt.Rows[j]["data_precision"]) + "," + n7.nvl(dt.Rows[j]["data_scale"]) + ") " + n7.nvl(dt.Rows[j]["nullable"]);
                        else
                            s = dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + " " + n7.nvl(dt.Rows[j]["nullable"]);
                        tn1 = tn.Nodes.Add(s);
                        tn1.ImageIndex = 1;
                        tn1.SelectedImageIndex = 1;
                    }
                    if (tn.Tag.ToString() == "T")
                    {
                        tn1 = tn.Nodes.Add("Indexes");
                        tn1.Tag = "I";
                        tn1.Nodes.Add(".");
                    }
                }
                else if ((tn.Tag.ToString() == "T" || tn.Tag.ToString() == "V") && n7.ProviderType == "ORA")
                {
                    if (showAllObj == 1)
                        q = "SELECT column_name, lower(data_type) data_type, data_length, data_precision, data_scale, decode(nullable,'N','not null',null) nullable"
                        + "    FROM all_tab_columns"
                        + "   WHERE owner = '" + tn.Text.Split('.')[0] + "'"
                        + "     AND table_name = '" + tn.Text.Split('.')[1] + "'"
                        + " ORDER BY column_id";
                    else
                        q = "SELECT column_name, lower(data_type) data_type, data_length, data_precision, data_scale, decode(nullable,'N','not null',null) nullable"
                        + "    FROM user_tab_columns"
                        + "   WHERE table_name = '" + tn.Text + "'"
                        + "   ORDER BY column_id";
                    n7.DataTableFill(dt, q);
                    for (int j = 0; j < dt.Rows.Count; j++)
                    {
                        if (dt.Rows[j]["data_type"].ToString() == "char" || dt.Rows[j]["data_type"].ToString() == "varchar" || dt.Rows[j]["data_type"].ToString() == "varchar2")
                            s = dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + "(" + dt.Rows[j]["data_length"].ToString() + ") " + n7.nvl(dt.Rows[j]["nullable"]);
                        else if (dt.Rows[j]["data_type"].ToString() == "number" && n7.nvl(dt.Rows[j]["data_precision"]) != "")
                            s = dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + "(" + n7.nvl(dt.Rows[j]["data_precision"]) + "," + n7.nvl(dt.Rows[j]["data_scale"]) + ") " + n7.nvl(dt.Rows[j]["nullable"]);
                        else
                            s = dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["data_type"].ToString() + " " + n7.nvl(dt.Rows[j]["nullable"]);
                        tn1 = tn.Nodes.Add(s);
                        tn1.ImageIndex = 1;
                        tn1.SelectedImageIndex = 1;
                    }
                    if (tn.Tag.ToString() == "T")
                    {
                        tn1 = tn.Nodes.Add("Indexes");
                        tn1.Tag = "I";
                        tn1.Nodes.Add(".");
                    }
                }
                else if ((tn.Tag.ToString() == "T" || tn.Tag.ToString() == "V") && n7.ProviderType == "MYSQL")
                {
                    r = new string[3];
                    r[0] = null;
                    if (tn.Text.IndexOf('.') > 0)
                    {
                        r[1] = tn.Text.Split('.')[0];
                        r[2] = tn.Text.Split('.')[1];
                    }
                    else
                    {
                        r[1] = null;
                        r[2] = tn.Text;
                    }
                    DbConnection cn = n7.CreateConnection();
                    cn.Open();
                    dt = cn.GetSchema("Columns", r);
                    cn.Close();

                    for (int j = 0; j < dt.Rows.Count; j++)
                    {
                        tn1 = tn.Nodes.Add(dt.Rows[j]["column_name"].ToString() + " " + dt.Rows[j]["column_type"].ToString());
                        tn1.ImageIndex = 1;
                        tn1.SelectedImageIndex = 1;
                    }
                }
                else if (tn.Tag.ToString() == "I" && n7.ProviderType == "SQL")
                {
                    DbConnection cn = n7.CreateConnection();
                    cn.Open();
                    dt.Clear();
                    string[] restrictions = { null, null, tn.Parent.Text };
                    dt = cn.GetSchema("IndexColumns", restrictions);
                    cn.Close();

                    if (dt.Rows.Count > 0)
                    {
                        s = "";
                        string indexName = "";
                        DataRow[] rw = dt.Select("table_name='" + tn.Parent.Text + "'", "index_name,ordinal_position");
                        for (int j = 0; j < rw.Length; j++)
                        {
                            if (indexName == rw[j]["index_name"].ToString())
                                s += ", " + rw[j]["column_name"].ToString();
                            else
                            {
                                if (j > 0)
                                {
                                    s += ")";
                                    tn1 = tn.Nodes.Add(s);
                                    tn1.ImageIndex = 2;
                                    tn1.SelectedImageIndex = 2;
                                }
                                s = rw[j]["index_name"].ToString() + " " + " (" + rw[j]["column_name"].ToString();
                                indexName = rw[j]["index_name"].ToString();
                            }
                        }
                        s += ")";
                        tn1 = tn.Nodes.Add(s);
                        tn1.ImageIndex = 2;
                        tn1.SelectedImageIndex = 2;
                    }
                }
                else if (tn.Tag.ToString() == "I" && n7.ProviderType == "ORA")
                {
                    if (showAllObj == 1)
                        q = "SELECT a.index_name, a.uniqueness, b.column_name"
                          + "  FROM all_indexes a, all_ind_columns b"
                          + " WHERE a.owner = b.index_owner"
                          + "   AND a.index_name=b.index_name"
                          + "   AND a.owner = '" + tn.Parent.Text.Split('.')[0] + "'"
                          + "   AND a.table_name='" + tn.Parent.Text.Split('.')[1] + "'"
                          + " ORDER BY a.index_name, b.column_position";
                    else
                        q = "SELECT a.index_name, a.uniqueness, b.column_name"
                          + "  FROM user_indexes a, user_ind_columns b"
                          + " WHERE a.index_name=b.index_name"
                          + "   AND a.table_name='" + tn.Parent.Text + "'"
                          + " ORDER BY a.index_name, b.column_position";
                    dt.Clear();
                    n7.DataTableFill(dt, q);
                    if (dt.Rows.Count > 0)
                    {
                        s = "";
                        string indexName = "";
                        for (int j = 0; j < dt.Rows.Count; j++)
                        {
                            if (indexName == dt.Rows[j]["index_name"].ToString())
                                s += ", " + dt.Rows[j]["column_name"].ToString();
                            else
                            {
                                if (j > 0)
                                {
                                    s += ")";
                                    tn1 = tn.Nodes.Add(s);
                                    tn1.ImageIndex = 2;
                                    tn1.SelectedImageIndex = 2;
                                }
                                s = (dt.Rows[j]["index_name"].ToString() + " " + dt.Rows[j]["uniqueness"].ToString().ToLower()).TrimEnd() + " (" + dt.Rows[j]["column_name"].ToString();
                                indexName = dt.Rows[j]["index_name"].ToString();
                            }
                        }
                        s += ")";
                        tn1 = tn.Nodes.Add(s);
                        tn1.ImageIndex = 2;
                        tn1.SelectedImageIndex = 2;
                    }
                }
                else
                {
                    q = "SELECT * FROM " + tn.Text + " WHERE 1=2";
                    n7.DataTableFill(dt, q);

                    for (int j = 0; j < dt.Columns.Count; j++)
                    {
                        s = dt.Columns[j].ColumnName + " " + dt.Columns[j].DataType.Name + (dt.Columns[j].AllowDBNull == true ? "" : "NOT NULL");
                        tn1 = tn.Nodes.Add(s);
                        tn1.ImageIndex = 1;
                        tn1.SelectedImageIndex = 1;
                    }
                }
            }

            dbTreeView.Cursor = null;
            SetStatusText("");
        }
        private void dbTreeView_ItemDrag(object sender, ItemDragEventArgs e)
        {
            TreeNode i = (TreeNode)e.Item;

            string textToDrag;

            if (n7.IsIn(n7.nvl(i.Tag), "T,V,S"))
            {
                textToDrag = i.Text;
                if (textToDrag.IndexOf('.') < 0)
                    textToDrag = "." + textToDrag;
                textToDrag += ".";
            }
            else
            {
                textToDrag = i.Parent.Text;
                if (textToDrag.IndexOf('.') < 0)
                    textToDrag = "." + textToDrag;
                textToDrag += "." + i.Text;
            }

            this.DoDragDrop(textToDrag, DragDropEffects.Copy);
        }

        private void sqlText_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
                e.Effect = DragDropEffects.Copy;
            else
                e.Effect = DragDropEffects.None;
        }
        private void sqlText_DragDrop(object sender, DragEventArgs e)
        {
            string[] v = e.Data.GetData(DataFormats.Text).ToString().Split('.');
            string s = "";

            if (v[2] == "")
            {
                // table
                s = v[0] + "." + v[1];
                if (s.StartsWith("."))
                    s = s.Substring(1);
                if (e.KeyState == 4)
                    s = BuildQuery(s, true);
            }
            else
                // field
                if (e.KeyState == 4)
                    s = v[1] + "." + v[2];
                else
                    s = v[2];

            if (!s.ToUpper().StartsWith("SELECT ") && s.Length > 0 && s.IndexOf(' ') >= 0)
                s = s.Split(' ')[0];

            Point p = sqlText.PointToClient(new Point(e.X, e.Y));
            int i = CharFromPos(sqlText, p);
            sqlText.Text = sqlText.Text.Substring(0, i) + s + sqlText.Text.Substring(i);
            sqlText.Focus();
            sqlText.SelectionStart = i + s.Length;
            sqlText.SelectionLength = 0;
            sqlText.ScrollToCaret();
        }
        private void sqlText_KeyPress(object sender, KeyPressEventArgs e)
        {
            sqlTextModified = true;
        }

        private void AddToHistory(string queryToAdd)
        {
            if (sqlTextModified && lastSql != queryToAdd)
                n7.FileWrite(Environment.GetEnvironmentVariable("APPDATA") + "\\FreeSql\\freeSQL.hst", queryToAdd.Replace("\r\n", ((char)1).ToString()) + "\r\n", true);
            lastSql = queryToAdd;
        }
        private string BeautifySelDel(string qtype, string sql, string moreSpaces)
        {
            if (!sql.StartsWith(qtype, StringComparison.OrdinalIgnoreCase)) return sql;

            int braket = 0;
            int braketBegins = 0;
            int betweenIndent = 0;
            string newSql = qtype + " ";
            string instruction = "";
            int k = 0, level = 0;
            int[] s_indentation = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
            int[] j_indentation = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
            int[] o_indentation = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
            string spaces;

            sql = sql.Replace("''", "{apex}");
            for (int i = 7; i < sql.Length; i++)
            {
                if (n7.SubString(sql, i, 1) == "'")
                {
                    newSql += sql.Substring(i, 1);
                    i++;
                    while (n7.SubString(sql, i, 1) != "'")
                    {
                        newSql += sql.Substring(i, 1);
                        i++;
                        if (i >= sql.Length)
                            break;
                    }
                    if (i >= sql.Length)
                        break;
                    newSql += sql.Substring(i, 1);
                    i++;
                }
                if (i >= sql.Length)
                    break;
                if (n7.SubString(sql, i, 1) == "(" && sql.Substring(i + 1).Trim().StartsWith(qtype, StringComparison.OrdinalIgnoreCase))
                {
                    level++;
                    braketBegins = braket;
                }
                else if (n7.SubString(sql, i, 1) == ")" && braket == braketBegins)
                    level--;
                else if (n7.SubString(sql, i, 1) == "(")
                    braket++;
                else if (n7.SubString(sql, i, 1) == ")")
                    braket--;

                if (sql.Substring(i).StartsWith(" BETWEEN ", StringComparison.OrdinalIgnoreCase))
                    betweenIndent = newSql.Length - newSql.LastIndexOf("\r\n") + 2;
                else if (sql.Substring(i).StartsWith(" AND ", StringComparison.OrdinalIgnoreCase) && betweenIndent > 0)
                {
                    newSql += "\r\n" + Space(betweenIndent) + " and";
                    k = betweenIndent;
                    betweenIndent = 0;
                    i = i + 4;
                }

                instruction = "";
                if (level >= 0)
                {
                    if (sql.Substring(i).StartsWith(" AND ", StringComparison.OrdinalIgnoreCase))
                        instruction = "AND";
                    else if (sql.Substring(i).StartsWith(" OR ", StringComparison.OrdinalIgnoreCase))
                        instruction = "OR";
                }
                if (instruction == "" && braket == 0 && level >= 0)
                {
                    if (sql.Substring(i).StartsWith("SELECT ", StringComparison.OrdinalIgnoreCase))
                        s_indentation[level] = newSql.Length - newSql.LastIndexOf("\r\n", newSql.Length) - 2;
                    else if (sql.Substring(i).StartsWith(" FROM ", StringComparison.OrdinalIgnoreCase))
                        instruction = "FROM";
                    else if (sql.Substring(i).StartsWith(" WHERE ", StringComparison.OrdinalIgnoreCase))
                        instruction = "WHERE";
                    else if (sql.Substring(i).StartsWith(" --"))
                        instruction = "--";
                    else if (sql.Substring(i).StartsWith(" AND ", StringComparison.OrdinalIgnoreCase))
                        instruction = "AND";
                    else if (sql.Substring(i).StartsWith(" OR ", StringComparison.OrdinalIgnoreCase))
                        instruction = "OR";
                    else if (sql.Substring(i).StartsWith(" ORDER BY ", StringComparison.OrdinalIgnoreCase))
                        instruction = "ORDER BY";
                    else if (sql.Substring(i).StartsWith(" GROUP BY ", StringComparison.OrdinalIgnoreCase))
                        instruction = "GROUP BY";
                    else if (sql.Substring(i).StartsWith(" HAVING ", StringComparison.OrdinalIgnoreCase))
                        instruction = "HAVING";
                    else if (sql.Substring(i).StartsWith(" JOIN ", StringComparison.OrdinalIgnoreCase) && j_indentation[level] == 0)
                        j_indentation[level] = newSql.Length - newSql.LastIndexOf("\r\n", newSql.Length) - 1;
                    else if (sql.Substring(i).StartsWith(" JOIN ", StringComparison.OrdinalIgnoreCase) && j_indentation[level] > 0)
                        instruction = "JOIN";
                    else if (sql.Substring(i).StartsWith(" LEFT JOIN ", StringComparison.OrdinalIgnoreCase) && j_indentation[level] > 0)
                        instruction = "LEFT JOIN";
                    else if (sql.Substring(i).StartsWith(" LEFT OUTER JOIN ", StringComparison.OrdinalIgnoreCase) && j_indentation[level] > 0)
                        instruction = "LEFT OUTER JOIN";
                    else if (sql.Substring(i).StartsWith(" RIGHT JOIN ", StringComparison.OrdinalIgnoreCase) && j_indentation[level] > 0)
                        instruction = "RIGHT JOIN";
                    else if (sql.Substring(i).StartsWith(" RIGHT OUTER JOIN ", StringComparison.OrdinalIgnoreCase) && j_indentation[level] > 0)
                        instruction = "RIGHT OUTER JOIN";
                    else if (sql.Substring(i).StartsWith(" ON ", StringComparison.OrdinalIgnoreCase) && j_indentation[level] > 0)
                        o_indentation[level] = newSql.Length - newSql.LastIndexOf("\r\n", newSql.Length) - 2;
                }

                if (instruction == "WHERE")
                {
                    j_indentation[level] = 0;
                    o_indentation[level] = 0;
                }
                if (instruction != "")
                {
                    if (instruction == "AND" && o_indentation[level] > 0)
                        spaces = Space(o_indentation[level]);
                    else if (instruction == "OR" && o_indentation[level] > 0)
                        spaces = Space(o_indentation[level]);
                    else if (j_indentation[level] > 0)
                        spaces = Space(j_indentation[level]);
                    else if (s_indentation[level] > 0)
                        spaces = Space(s_indentation[level] + 6 - instruction.Length);
                    else
                        spaces = Space(6 - instruction.Length);
                    if (spaces == "") spaces = " ";
                    if (level == 0) spaces += moreSpaces;

                    k = spaces.Length;
                    i += instruction.Length + 1;
                    newSql += "\r\n" + spaces + instruction;
                }
                else if (k > 70 && n7.RevSubString(newSql, 1) == " ")
                {
                    newSql += "\r\n       " + Space(s_indentation[level]);
                    k = 0;
                }

                if (sql.Substring(i, 1) == "+" && braket == 0 && instruction == "SELECT")
                {
                    if (sql.Substring(i - 1, 1) != " ") newSql += " ";
                    newSql += "+\r\n      " + Space(s_indentation[level]);
                    k = 0;
                }
                else if (sql.Substring(i, 1) == "|" && braket == 0 && instruction == "SELECT")
                {
                    if (sql.Substring(i - 1, 1) != " ") newSql += " ";
                    newSql += "||\r\n      " + Space(s_indentation[level]);
                    k = 0;
                    i++;
                }
                else
                    newSql += sql.Substring(i, 1);

                k++;
            }

            newSql = newSql.Replace("select ", "SELECT ");
            newSql = newSql.Replace("{apex}", "''");
            return newSql;
        }
        private string BeautifyInsert(string sql)
        {
            if (!sql.StartsWith("INSERT", StringComparison.OrdinalIgnoreCase)) return sql;

            bool valuesClause = false;
            string newSql = "INSERT INTO ";
            sql = sql.Replace("''", "{apex}");
            for (int i = 12; i < sql.Length; i++)
            {
                if (n7.SubString(sql, i, 1) == "'")
                {
                    newSql += sql.Substring(i, 1);
                    i++;
                    while (n7.SubString(sql, i, 1) != "'")
                    {
                        newSql += sql.Substring(i, 1);
                        i++;
                    }
                    newSql += sql.Substring(i, 1);
                    i++;
                }
                if (i >= sql.Length)
                    break;
                if (sql.Substring(i).StartsWith("SELECT ", StringComparison.OrdinalIgnoreCase) && valuesClause == false)
                {
                    newSql += "\r\n" + BeautifySelDel("SELECT", sql.Substring(i), "");
                    break;
                }
                else if (sql.Substring(i).StartsWith(" VALUES ", StringComparison.OrdinalIgnoreCase))
                {
                    newSql += "\r\nVALUES";
                    i = i + 6;
                    valuesClause = true;
                }
                else
                    newSql += sql.Substring(i, 1);
            }

            newSql = newSql.Replace("{apex}", "''");
            return newSql;
        }
        private string BeautifyUpdate(string sql)
        {
            if (!sql.StartsWith("UPDATE", StringComparison.OrdinalIgnoreCase)) return sql;

            bool setStatement = false;
            string newSql = "UPDATE ";
            sql = sql.Replace("''", "{apex}");
            for (int i = 7; i < sql.Length; i++)
            {
                if (n7.SubString(sql, i, 1) == "'")
                {
                    newSql += sql.Substring(i, 1);
                    i++;
                    while (n7.SubString(sql, i, 1) != "'")
                    {
                        newSql += sql.Substring(i, 1);
                        i++;
                    }
                    newSql += sql.Substring(i, 1);
                    i++;
                }
                if (i >= sql.Length)
                    break;
                if (n7.SubString(sql, i, 1) == "(" && sql.Substring(i + 1).Trim().StartsWith("SELECT", StringComparison.OrdinalIgnoreCase))
                {
                    int braket = 0, e = sql.Length;
                    bool apex = false;
                    for (int k = i; k < sql.Length; k++)
                    {
                        if (n7.SubString(sql, k, 1) == "'" && apex == false)
                            apex = true;
                        else if (n7.SubString(sql, k, 1) == "'" && apex == true)
                            apex = false;
                        if (n7.SubString(sql, k, 1) == "(" && apex == false)
                            braket++;
                        if (n7.SubString(sql, k, 1) == ")" && apex == false)
                            braket--;
                        if (n7.SubString(sql, k, 1) == ")" && braket == 0)
                        {
                            e = k;
                            break;
                        }
                    }
                    int l = newSql.LastIndexOf("\r\n");
                    l = newSql.Length - l - 1;
                    newSql += "(" + BeautifySelDel("SELECT", n7.SubString(sql, i + 1, e - i - 1), "".PadRight(l)).TrimEnd() + ")";
                    i = e;
                }
                else if (sql.Substring(i).StartsWith(" SET ", StringComparison.OrdinalIgnoreCase))
                {
                    newSql += "\r\n   SET";
                    i = i + 3;
                    setStatement = true;
                }
                else if (sql.Substring(i).StartsWith(" WHERE ", StringComparison.OrdinalIgnoreCase))
                {
                    newSql += "\r\n WHERE";
                    i = i + 5;
                    setStatement = false;
                }
                else if (sql.Substring(i).StartsWith(" AND", StringComparison.OrdinalIgnoreCase))
                {
                    newSql += "\r\n   AND";
                    i = i + 3;
                }
                else if (sql.Substring(i).StartsWith(" OR", StringComparison.OrdinalIgnoreCase))
                {
                    newSql += "\r\n    OR";
                    i = i + 2;
                }
                else if (sql.Substring(i, 1) == "," && setStatement == true)
                {
                    newSql += ",\r\n       ";
                    i = i + 1;
                }
                else
                    newSql += sql.Substring(i, 1);
            }

            newSql = newSql.Replace("{apex}", "''");
            return newSql;
        }
        private int BuildCreateTable(string tableName, string[] cols)
        {
            string q = "CREATE TABLE " + tableName + "(";
            for (int i = 0; i < cols.Length; i++)
            {
                string pKey = "";
                if (cols[i].IndexOf('*') >= 0)
                    pKey = "PRIMARY KEY";
                cols[i] = n7.StringFilter(cols[i], netlib7.StringType.CharsAndNumbersAndSpaces, "_");
                cols[i] = cols[i].Replace(' ', '_');
                cols[i] = n7.SubString(cols[i], 0, 20);
                q += cols[i] + " VARCHAR(255) " + pKey + ", ";
            }
            q = n7.RTrunc(q, 2) + ")";
            if (n7.ExecNonQuery(q) < 0)
            {
                MessageBox.Show(n7.ErrorMessage);
                return -1;
            }
            else
            {
                if (q.IndexOf("PRIMARY KEY") >= 0)
                    return 1;
                else
                    return 0;
            }
        }
        private string BuildQuery(string objectFullName, bool useObjectName)
        {
            string e1, e2;
            if (n7.ProviderType == "ORA")
            {
                e1 = "\"";
                e2 = "\"";
            }
            else
            {
                e1 = "[";
                e2 = "]";
            }

            string userName, objectName;
            if (objectFullName.IndexOf('.') > 0)
            {
                userName = objectFullName.Split('.')[0];
                objectName = objectFullName.Split('.')[1];
            }
            else
            {
                userName = "";
                objectName = objectFullName;
            }
            if (n7.StringFilter(objectName, netlib7.StringType.CharsAndNumbers, "_") != objectName || (n7.ProviderType == "ORA" && objectName.ToUpper() != objectName))
            {
                objectName = e1 + objectName + e2;
                if (userName != "")
                    objectFullName = userName + "." + objectName;
                else
                    objectFullName = objectName;
            }

            DataTable dt = new DataTable();
            n7.DataTableFill(dt, "select * from " + objectFullName + " where 1=2");

            string fieldList = "", colName;
            if (dt.Columns.Count > 0)
            {
                int k = 1;
                for (int i = 0; i < dt.Columns.Count; i++)
                {
                    colName = dt.Columns[i].ColumnName;
                    if (n7.StringFilter(colName, netlib7.StringType.CharsAndNumbers, "_") != colName || (n7.ProviderType == "ORA" && colName.ToUpper() != colName))
                        colName = e1 + colName + e2;

                    if (useObjectName)
                        fieldList += objectName + "." + colName;
                    else
                        fieldList += colName;
                    if (i < dt.Columns.Count - 1)
                    {
                        fieldList += ", ";
                        if (Convert.ToDouble(fieldList.Length / 70.0) >= k)
                        {
                            fieldList += "\r\n       ";
                            k++;
                        }
                    }
                }
            }

            string q = "SELECT " + fieldList + "\r\n  FROM " + objectFullName;
            string limit = n7.nvl(Registry.GetValue("HKEY_CURRENT_USER\\Software\\FreeSQL", "optMaxRecords", null), "100");

            if (limit != "0")
            {
                if (n7.ProviderType == "SQL")
                    q = "SELECT TOP " + limit + " " + q.Substring(7);
                else if (n7.ProviderType == "ORA")
                    q += "\r\n WHERE ROWNUM <= " + limit;
                else if (n7.ProviderType == "MYSQL")
                    q += "\r\n LIMIT " + limit;
            }
            return q;
        }
        private void ChooseDB()
        {
            frmConnect c = new frmConnect();
            c.ShowDialog();

            if (c.ConnectionString != "")
            {
                n7.ProviderName = c.ProviderName;
                n7.ConnectionString = c.ConnectionString;

                this.Text = Application.ProductName + " " + Application.ProductVersion;
                if (n7.ConnectionString != "")
                {
                    string s = n7.GetConnectionProperty("data source");
                    if (s == "") s = n7.GetConnectionProperty("server");
                    this.Text += " - " + n7.GetConnectionProperty("user id") + " on " + s;
                }
                if (queryOnlyMode)
                    this.Text += " (query-only mode)";
            }
        }
        private void EditCell(int rowIndex, int columnIndex)
        {
            if (rowIndex < 0)
                return;

            frmEdit t = new frmEdit();
            t.textValue = dataGrid.Rows[rowIndex].Cells[columnIndex].Value.ToString();
            t.readOnly = queryOnlyMode;
            t.ShowDialog();
            if (!queryOnlyMode)
                dataGrid.Rows[rowIndex].Cells[columnIndex].Value = t.textValue;
        }
        private string QueryCleanUp(string sql, bool removeCRLF)
        {
            sql = sql.Replace("\t", " ");
            string[] v = sql.Split('\r', '\n');
            sql = "";
            for (int i = 0; i < v.Length; i++)
            {
                if (v[i].Trim() != "" && v[i].Trim().StartsWith("--") == false)
                {
                    sql += v[i];
                    if (removeCRLF)
                        sql += " ";
                    else
                        sql += "\r\n";
                }
            }
            sql = sql.Trim();
            sql = sql.Replace("''", "{apex}");

            for (int i = 0; i < sql.Length - 1; i++)
            {
                if (n7.SubString(sql, i, 1) == "'")
                {
                    i++;
                    while (n7.SubString(sql, i, 1) != "'")
                        i++;
                    i++;
                }
                if (i >= sql.Length)
                    break;
                if (i < sql.Length && sql.Substring(i, 1) == " " && sql.Substring(i + 1, 1) == " ")
                {
                    sql = sql.Remove(i, 1);
                    i--;
                }
            }

            sql = sql.Replace("{apex}", "''");
            if (sql.EndsWith(";")) sql = n7.RTrunc(sql, 1);
            return sql;
        }
        private void ResetTree()
        {
            dbTreeView.Nodes.Clear();
            TreeNode tn;
            tn = dbTreeView.Nodes.Add("Tables");
            tn.Nodes.Add(".");
            tn = dbTreeView.Nodes.Add("Views");
            tn.Nodes.Add(".");
            tn = dbTreeView.Nodes.Add("Synonyms");
            tn.Nodes.Add(".");
        }
        private void SetStatusText(string infoMessage)
        {
            if (infoMessage == "")
                infoMessage = "Ready";
            statusBar.Items["statusLabel"].Text = infoMessage;

            Application.DoEvents();
        }
        private string Space(int length)
        {
            if (length > 0)
                return new String(' ', length);
            else
                return "";
        }

        #region API functions
        [DllImport("user32", CharSet = CharSet.Auto, EntryPoint = "SendMessage")]
        private extern static int SendMessageInt(IntPtr handle, int msg, int wParam, int lParam);
        public static int CharFromPos(System.Windows.Forms.TextBox txt, Point pt)
        {
            unchecked
            {
                // Convert the point into a DWord with horizontal position
                // in the loword and vertical position in the hiword:
                int xy = (pt.X & 0xFFFF) + ((pt.Y & 0xFFFF) << 16);
                // Get the position from the text box.
                int res = SendMessageInt(txt.Handle, 0x00D7, 0, xy);
                // the Platform SDK appears to be incorrect on this matter.
                // the hiword is the line number and the loword is the index
                // of the character on this line
                int lineNumber = ((res & 0xFFFF) >> 16);
                int charIndex = (res & 0xFFFF);

                // Find the index of the first character on the line within 
                // the control:
                int lineStartIndex = SendMessageInt(txt.Handle, 0x00BB, lineNumber, 0);
                // Return the combined index:
                return lineStartIndex + charIndex;
            }
        }
        #endregion

    }
}